import shutil
from pathlib import Path
from typing import Union

import tomllib

from ramonak import PACKAGES_PATH
from ramonak.packages import NEXUS_PATH
from ramonak.packages.utils import (
    fetch_unzip,
    get_package_id_parts,
    get_package_versions,
    local_package_exists,
    retrieve_package_url,
)


def require(package_id: str) -> Path:
    package_author, package_name, package_version = get_package_id_parts(package_id)

    if "==" not in package_id:
        package_version = get_package_versions(package_author, package_name)[-1]["id"]
        print(
            f"Required '{package_id}=={package_version}'...",
            end=" ",
        )
    else:
        print(f"Required package '{package_id}'...", end=" ")

    package_path = Path(
        PACKAGES_PATH, package_author, package_name, str(package_version)
    )

    if local_package_exists(package_id):
        print("Already satisfied")
        return package_path
    else:
        print("Downloading...")

    file_url = retrieve_package_url(package_author, package_name, package_version)

    fetch_unzip(
        file_url,
        package_path,
    )

    print(
        f"The package '{package_author}/{package_name}=={package_version}' has been installed successfully"
    )

    return package_path


def remove(package_id: str):
    removable_path = ""

    author, name, version = get_package_id_parts(package_id)

    if "==" not in package_id:
        print(
            "Removing the local metapackage '{}/{}'...".format(author, name),
            end=" ",
        )
        removable_path = Path(PACKAGES_PATH, author, name)
    else:
        print(
            "Removing the local package '{}/{}=={}'...".format(author, name, version),
            end=" ",
        )
        removable_path = Path(PACKAGES_PATH, author, name, version)

    try:
        shutil.rmtree(removable_path)
    except FileNotFoundError:
        raise Exception("The package doesn't exist in the local storage")
    else:
        print("OK")


def purge():
    print("Removing all the local packages...", end=" ")

    shutil.rmtree(PACKAGES_PATH)
    PACKAGES_PATH.mkdir(parents=True)

    print("OK")


def info(package_id) -> Union[str, dict]:
    author, name, version = get_package_id_parts(package_id)
    package_file = str(Path(NEXUS_PATH, author, name)) + ".toml"
    descriptor_text = open(package_file, "r", encoding="utf8").read()
    descriptor_data = tomllib.loads(descriptor_text)

    if not version:
        print("type", "=", "metapackage")
    else:
        print("type", "=", "package")

    for key, value in descriptor_data["package_info"].items():
        print(key, "=", value)

    if not version:
        versions = ",".join(v["id"] for v in descriptor_data["versions"])
        print(f"versions = [{versions}]")
    else:
        version = next(v for v in descriptor_data["versions"] if v["id"] == version)

        for key, value in version.items():
            print(f"version.{key} = {value}")



from ramonak.packages.actions import info
from ramonak.stemmer import FlexionStatStemmer

stemmer = FlexionStatStemmer()

info("@alerus/flexistat_data==20250111_125624")


while True:
    word = input("Word to stem: ")

    if word == "exit":
        break

    print(stemmer.stem_word(word))



import decimal

def round_down(value, decimals=0):
    with decimal.localcontext() as ctx:
        d = decimal.Decimal(value)
        ctx.rounding = decimal.ROUND_DOWN
        return round(d, decimals)



from ramonak.stemmer.flexistat import FlexionStatStemmer


def test_flexistat_stemmer():
    stemmer = FlexionStatStemmer()

    assert stemmer.stem_words(["лесам", "лесу", "лесамі"]) == (["лес"] * 3)


def test_flexistat_stemmer_u():
    stemmer = FlexionStatStemmer()

    assert stemmer.stem_word("ўдот") == "удот"



import itertools
import re
import string

word_punct = string.punctuation + "…"
sent_punct = ".?!…"

re_word_tokenize = re.compile(r"[{}]+".format(re.escape(word_punct)))
re_word_tokenize_keep = re.compile(r"([{}]+)".format(re.escape(word_punct)))

re_sent_tokenize_keep = re.compile(
    r"([^{sent_punct}]+[{sent_punct}]+)".format(sent_punct=re.escape(sent_punct))
)


def word_tokenize(text: str, remove_punct=False) -> list[str]:
    regex = re_word_tokenize_keep

    if remove_punct:
        regex = re_word_tokenize

    result = []

    for sent_parts in regex.split(text):
        result.append(sent_parts.split())

    return list(itertools.chain(*result))


def sent_tokenize(text: str) -> list[str]:
    result = []

    for sentence in re_sent_tokenize_keep.split(text):
        sentence = sentence.strip()

        if not sentence:
            continue

        result.append(sentence)

    return result



import pytest

from ramonak.stemmer.base import Stemmer
from ramonak.stopwords import STOP_WORDS, clean_stop_words

fix_lang_phenomenons = Stemmer.fix_lang_phenomenons


specific_words = ["мы", "аднак", "безумоўна"]


@pytest.mark.parametrize("word", specific_words)
def test_check_words(word):
    assert word in STOP_WORDS


def test_clean_stop_words_in_list():
    assert clean_stop_words(["не", "пайшоў", "а", "паехаў"]) == ["пайшоў", "паехаў"]


def test_clean_stop_words_in_str():
    assert clean_stop_words("не пайшоў, а паехаў") == "пайшоў, паехаў"



import re


class Stemmer:
    def stem_word(self, word):
        raise NotImplementedError()

    @staticmethod
    def fix_lang_phenomenons(word: str) -> str:
        # region dzekannie, tsekannie, soft + jvowel = hard + jvowel
        vowel_pairs = {
            "е": "э",
            "ё": "о",
            "ю": "у",
            "я": "а",
            "і": "ы",
        }

        for jvowel, vowel in vowel_pairs.items():
            word = re.sub("дз" + jvowel, "д" + vowel, word)
            word = re.sub("ц" + jvowel, "т" + vowel, word)

            word = re.sub("дзв" + jvowel, "дв" + vowel, word)
            word = re.sub("цв" + jvowel, "тв" + vowel, word)
        # endregion

        # region Remove ў at the beginning
        word = re.sub(r"^ў", "у", word)
        # endregion

        return word

    def stem_words(self, words: list[str]) -> list[str]:
        result = []

        for word in words:
            result.append(self.stem_word(word))

        return result



import io
import re
import zipfile
from pathlib import Path
from typing import Tuple

import requests
import tomllib
from tqdm import tqdm

from ramonak import PACKAGES_PATH
from ramonak.packages import NEXUS_PATH


def fetch_unzip(zip_file_url: str, destination_dir: str) -> Path:
    Path(destination_dir).mkdir(exist_ok=True, parents=True)
    bio = io.BytesIO()

    response = requests.get(zip_file_url, stream=True)
    with tqdm.wrapattr(
        bio,
        "write",
        miniters=1,
        desc=zip_file_url.split("/")[-1],
        total=int(response.headers.get("content-length", 0)),
    ) as fout:
        for chunk in response.iter_content(chunk_size=4096):
            fout.write(chunk)

    z = zipfile.ZipFile(bio)
    z.extractall(destination_dir)


def package_path(package_id: str) -> Path:
    author, name, version = get_package_id_parts(package_id)
    return Path(PACKAGES_PATH, author, name, version)


def local_package_exists(package_name: str) -> bool:
    package_dir = package_path(package_name)

    if package_dir.exists() and any(package_dir.iterdir()):
        return True
    else:
        return False


def get_package_id_parts(package: str) -> Tuple[str, str, str]:
    package = re.sub(r"\s", "", package)

    package_version = ""
    package_name = ""
    package_author = ""

    if "==" in package:  # Installing exact version
        _, package_version = package.split("==")
        package_author, package_name = package.split("==")[0].split("/")
    elif package.count("/") == 1:  # Get latest version
        package_author, package_name = package.split("/")
    else:
        raise Exception(
            "Wrong package name. At least author and package name must be present"
        )

    return package_author, package_name, package_version


def get_package_versions(package_author, package_name) -> list:
    package_file = str(Path(NEXUS_PATH, package_author, package_name)) + ".toml"
    package_dict = tomllib.loads(open(package_file, "r", encoding="utf8").read())

    return package_dict["versions"]


def retrieve_package_url(package_author, package_name, package_version) -> str:
    package_file = str(Path(NEXUS_PATH, package_author, package_name)) + ".toml"
    package_dict = tomllib.loads(open(package_file, "r", encoding="utf8").read())

    for version in package_dict["versions"]:
        if version["id"] == package_version:
            return version["url"]

    raise Exception(
        "No such package version found: {}/{}=={}".format(
            package_author, package_name, package_version
        )
    )
